#ifdef WIN32
#include <windows.h>
#elif defined(MAC)
#include <mach-o/dyld.h>
#include <boost/cstdint.hpp>
#else
#include <unistd.h>
#endif

#include "paths.h"

boost::filesystem::path Paths::getExecPath()
{
#ifdef MAC
    char path[4096];
    uint32_t size = sizeof(path);
    if (_NSGetExecutablePath(path, &size) == 0) {
        return boost::filesystem::path(path).parent_path();
    }
#elif defined(WIN32)
    char path[2048];
    GetModuleFileNameA(NULL, path, sizeof(path));
    return boost::filesystem::path(path).parent_path();
#else
    char path[4096];
    size_t size = readlink("/proc/self/exe", path, sizeof(path));
    if (size > 0) {
        return boost::filesystem::path(path).parent_path();
    }
#endif
    return boost::filesystem::path();
}
