#include <QHBoxLayout>
#include <QGridLayout>
#include <QItemSelectionModel>
#include <QAction>

#include "messagesmodel.h"

#include "messagehistorywidget.h"

MessageHistoryWindow::MessageHistoryWindow(QWidget * parent)
    : QWidget(parent)
{
    QHBoxLayout * mainLayout = new QHBoxLayout(this);
    m_splitter = new QSplitter();
    m_messageList = new QListView();
    m_splitterCollapser = new SplitterCollapser(m_splitter, this);
    QWidget * rightWidget = new QWidget();
    QVBoxLayout * messageLayout = new QVBoxLayout();
    QHBoxLayout * buttonsLayout = new QHBoxLayout();
    m_dateLabel = new QLabel();
    m_webView = new QWebView();
    m_leftButton = new QToolButton();
    m_leftButton->setArrowType(Qt::LeftArrow);
    m_rightButton = new QToolButton();
    m_rightButton->setArrowType(Qt::RightArrow);

    m_rightButton->setEnabled(false);
    m_leftButton->setEnabled(false);

    connect(m_leftButton, SIGNAL(clicked(bool)), this, SLOT(m_leftClicked()));
    connect(m_rightButton, SIGNAL(clicked(bool)), this, SLOT(m_rightClicked()));

    buttonsLayout->addWidget(m_leftButton);
    buttonsLayout->addStretch();
    buttonsLayout->addWidget(m_rightButton);
    messageLayout->addWidget(m_dateLabel);
    messageLayout->addWidget(m_webView);
    messageLayout->addLayout(buttonsLayout);
    rightWidget->setLayout(messageLayout);
    m_splitter->addWidget(m_messageList);
    m_splitter->addWidget(rightWidget);
    mainLayout->addWidget(m_splitter);
    setLayout(mainLayout);

    QAction * quitAction = new QAction(this);
    quitAction->setShortcut(Qt::Key_Escape);
    quitAction->setShortcutContext(Qt::ApplicationShortcut);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));
    addAction(quitAction);
}

MessageHistoryWindow::~MessageHistoryWindow()
{
}

void MessageHistoryWindow::setModel(MessagesModel * model)
{
    QItemSelectionModel * selectionModel = new QItemSelectionModel(model);
    m_messageList->setModel(model);
    m_messageList->setSelectionModel(selectionModel);
    connect(m_messageList->selectionModel(), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(m_messageChanged(const QModelIndex &, const QModelIndex &)));
    m_model = model;
}

void MessageHistoryWindow::m_leftClicked()
{
    QModelIndex currentIndex(m_messageList->selectionModel()->currentIndex());
    m_messageList->selectionModel()->setCurrentIndex(m_model->index(currentIndex.row() + 1, currentIndex.column()), QItemSelectionModel::SelectCurrent);
}

void MessageHistoryWindow::m_rightClicked()
{
    QModelIndex currentIndex(m_messageList->selectionModel()->currentIndex());
    m_messageList->selectionModel()->setCurrentIndex(m_model->index(currentIndex.row() - 1, currentIndex.column()), QItemSelectionModel::SelectCurrent);
}

void MessageHistoryWindow::m_messageChanged(const QModelIndex & to, const QModelIndex & /*from*/)
{
    if (!to.isValid()) {
        m_dateLabel->setText("");
        m_webView->setHtml("");
        m_rightButton->setEnabled(false);
        m_leftButton->setEnabled(false);
    }
    m_dateLabel->setText(tr("Created: ") + m_model->dataDate(to));
    m_webView->setHtml(m_model->dataText(to));
    if (to.row() == 0) {
        if (m_rightButton->isEnabled())
            m_rightButton->setEnabled(false);
    } else {
        if (!m_rightButton->isEnabled())
            m_rightButton->setEnabled(true);
    }
    if (to.row() == m_model->rowCount() - 1) {
        if (m_leftButton->isEnabled())
            m_leftButton->setEnabled(false);
    } else {
        if (!m_leftButton->isEnabled())
            m_leftButton->setEnabled(true);
    }
}

void MessageHistoryWindow::showHistory()
{
    show();
}

void MessageHistoryWindow::showLast()
{
    if (!isVisible()) {
        QModelIndex currentIndex(m_messageList->selectionModel()->currentIndex());
        m_messageList->selectionModel()->setCurrentIndex(m_model->index(0, currentIndex.column()), QItemSelectionModel::SelectCurrent);
        show();
    }
}
