#include <QDateTime>

#include "message.h"
#include "messagesmodel.h"

MessagesModel::MessagesModel(QObject * parent, const boost::filesystem::path & dataDir)
    : QAbstractListModel(parent),
      m_history(dataDir)
{
}

MessagesModel::~MessagesModel()
{
}

int MessagesModel::rowCount(const QModelIndex &) const
{
    return m_history.count();
}

QVariant MessagesModel::data(const QModelIndex & index, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();
    if (index.column() != 0)
        return QVariant();
    if (index.row() < 0 ||
        index.row() >= static_cast<int>(m_history.count()))
        return QVariant();
    const Message & message(m_history.get(m_history.count() - index.row() - 1));
    QString res(QDateTime::fromTime_t(message.created()).toString(Qt::DefaultLocaleShortDate));
    res += " ";
    res += QString::fromUtf8(message.text().c_str());
    return res;
}

QString MessagesModel::dataDate(const QModelIndex & index) const
{
    if (index.column() != 0)
        return "";
    if (index.row() < 0 ||
        index.row() >= static_cast<int>(m_history.count()))
        return "";
    const Message & message(m_history.get(m_history.count() - index.row() - 1));
    return QDateTime::fromTime_t(message.created()).toString(Qt::DefaultLocaleShortDate);
}

QString MessagesModel::dataText(const QModelIndex & index) const
{
    if (index.column() != 0)
        return "";
    if (index.row() < 0 ||
        index.row() >= static_cast<int>(m_history.count()))
        return "";
    const Message & message(m_history.get(m_history.count() - index.row() - 1));
    return QString("<pre>") + QString::fromUtf8(message.text().c_str()) + "</pre>";
}

void MessagesModel::addMessage(const Message & message)
{
    size_t pos = m_history.add(message);
    m_history.save();
    int from = m_history.count() - pos - 1;
    if (from < 0)
        from = 0;
    int to = m_history.count() - pos;
    if (to >= static_cast<int>(m_history.count()))
        to = m_history.count() - 1;
    emit dataChanged(createIndex(from, 0), createIndex(to, 0));
}
